###########################################################################
# Container Definition
###########################################################################

locals {
  signalfx_agent_daemon_container_definitions = [
    {
      name = "signalfx-agent"
      image     = "${data.aws_caller_identity.current.account_id}.dkr.ecr.${data.aws_region.current.name}.amazonaws.com/${var.client}/${var.envname}-signalfx-agent:${var.tag}"

      essential = "true"
      cpu       = var.signalfx_cpu
      memory    = var.signalfx_memory
      logConfiguration = {
        logDriver = "awslogs"
        options = {
          awslogs-group         = aws_cloudwatch_log_group.signalfx_agent_daemon.name
          awslogs-region        = data.aws_region.current.name
          awslogs-stream-prefix = "${var.envname}-signalfx-agent"
        }
      }
      networkMode  = "host"
      portMappings = []
      environment = [
        {
          name  = "ACCESS_TOKEN"
          value = var.signalfx_api_token
        },
        {
          name  = "INGEST_URL"
          value = "https://ingest.${var.signal_fx_realm}.signalfx.com"
        },
        {
          name  = "API_URL"
          value = "https://api.${var.signal_fx_realm}.signalfx.com"
        },
        {
          name  = "SIGNALFX_NODE"
          value = "${var.envname}-agent-daemon"
        },
        {
          name  = "SIGNALFX_MODE"
          value = var.signalfx_switch
        },
        {
          name  = "SIGNALFX_S3_CONFIG_STORE"
          value = "${var.client}-${var.envname}-signalfx-store/ecs/ecsdaemon"
        },
      ]
      mountPoints = [
        {
          readOnly      = "true"
          containerPath = "/hostfs"
          sourceVolume  = "hostfs"
        },
        {
          readOnly      = "true"
          containerPath = "/var/run/docker.sock"
          sourceVolume  = "docker-socket"
        },
        {
          readOnly      = "true"
          containerPath = "/etc/passwd"
          sourceVolume  = "etc-passwd"
        },
      ]
    },
  ]
}

###########################################################################
# TASK Definition
###########################################################################

resource "aws_ecs_task_definition" "signalfx_agent_daemon" {
  family        = "${var.envname}-signalfx-agent-daemon"
  network_mode  = "bridge"
  task_role_arn = aws_iam_role.signalfx_agent.arn


  # Nasty regex to workaround https://github.com/hashicorp/terraform/issues/17033
  container_definitions = replace(
    replace(
      jsonencode(local.signalfx_agent_daemon_container_definitions),
      "/\"([0-9]+\\.?[0-9]*|true|false)\"/",
      "$1",
    ),
    "/\"value\":([0-9]+\\.?[0-9]*|true|false)/",
    "\"value\":\"$1\"",
  )

  volume {
    name      = "hostfs"
    host_path = "/"
  }

  volume {
    name      = "docker-socket"
    host_path = "/var/run/docker.sock"
  }

  volume {
    name      = "etc-passwd"
    host_path = "/etc/passwd"
  }
}

###########################################################################
# ECS Service
###########################################################################

resource "aws_ecs_service" "signalfx_agent_daemon" {
  name                = "${var.envname}-signalfx-agent-daemon"
  cluster             = var.cluster
  task_definition     = aws_ecs_task_definition.signalfx_agent_daemon.arn
  scheduling_strategy = "DAEMON"
}

###########################################################################
# Cloudwatch
###########################################################################

resource "aws_cloudwatch_log_group" "signalfx_agent_daemon" {
  name              = "${var.envname}-signalfx-agent-daemon"
  retention_in_days = var.log_retention
}

resource "aws_cloudwatch_log_stream" "signalfx_agent_daemon" {
  name           = "${var.envname}-singalfx-daemon"
  log_group_name = aws_cloudwatch_log_group.signalfx_agent_daemon.name
}

###########################################################################
# S3 Object Store (Config)
###########################################################################
resource "aws_s3_bucket" "signalfx_config_store" {
  bucket = "${var.client}-${var.envname}-signalfx-store"
  acl    = "private"
}

###########################################################################
# S3 Object Store (Object)
###########################################################################

resource "null_resource" "signalfx_object_push" {
  provisioner "local-exec" {
    command = "aws s3 sync ${path.module}/files/signalfx/config s3://${aws_s3_bucket.signalfx_config_store.id}"
  }
}

